/*
 * Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.oracle.truffle.r.ffi.impl.interop;

import com.oracle.truffle.api.interop.CanResolve;
import com.oracle.truffle.api.interop.MessageResolution;
import com.oracle.truffle.api.interop.Resolve;
import com.oracle.truffle.api.interop.TruffleObject;
import com.oracle.truffle.api.nodes.Node;
import com.oracle.truffle.r.ffi.impl.common.JavaUpCallsRFFIImpl.VectorWrapper;
import com.oracle.truffle.r.runtime.data.CharSXPWrapper;
import com.oracle.truffle.r.runtime.ffi.DLL;
import com.oracle.truffle.r.runtime.ffi.DLL.DLLInfo;
import com.oracle.truffle.r.runtime.interop.RObjectNativeWrapper;

@MessageResolution(receiverType = DLL.DLLInfo.class)
public class DLLInfoMR {
    @CanResolve
    public abstract static class DLLInfolCheck extends Node {

        protected static boolean test(TruffleObject receiver) {
            return receiver instanceof DLL.DLLInfo;
        }
    }

    @Resolve(message = "TO_NATIVE")
    public abstract static class AsPointerNode extends Node {
        protected Object access(Object receiver) {
            return new RObjectNativeWrapper((DLLInfo) receiver);
        }
    }

    @Resolve(message = "READ")
    public abstract static class ReadNode extends Node {
        public Object access(Object receiver, Object index) {
            DLLInfo dllInfo = (DLLInfo) receiver;
            int i = ((Number) index).intValue();
            CharSXPWrapper res;
            switch (i) {
                case 0:
                    res = dllInfo.pathSXP;
                    break;
                case 1:
                    res = dllInfo.nameSXP;
                    break;
                default:
                    throw new IndexOutOfBoundsException("Index can be 0 or 1");
            }

            return VectorWrapper.get(res);
        }
    }
}
